@@ -10,15 +10,35 @@ CircuitPython module for the MMA8451 3 axis accelerometer.  See
 examples/simpletest.py for a demo of the usage.
 
 * Author(s): Tony DiCola
+
+Implementation Notes
+--------------------
+
+**Hardware:**
+
+* Adafruit `Triple-Axis Accelerometer - ±2/4/8g @ 14-bit - MMA8451
+  <https://www.adafruit.com/product/2019>`_
+
+
+**Software and Dependencies:**
+
+* Adafruit CircuitPython firmware for the supported boards:
+  https://circuitpython.org/downloads
+* Adafruit's Bus Device library:
+  https://github.com/adafruit/Adafruit_CircuitPython_BusDevice
+
 """
-try:
-    import struct
-except ImportError:
-    import ustruct as struct
+import struct
 
 from micropython import const
 
-import adafruit_bus_device.i2c_device as i2c_device
+from adafruit_bus_device import i2c_device
+
+try:
+    from typing import Optional, Tuple
+    from busio import I2C
+except ImportError:
+    pass
 
 
 __version__ = "0.0.0-auto.0"
@@ -64,17 +84,42 @@ DATARATE_1_56HZ = 0b111  # 1.56Hz
 
 class MMA8451:
     """MMA8451 accelerometer.  Create an instance by specifying:
-    - i2c: The I2C bus connected to the sensor.
 
-    Optionally specify:
-    - address: The I2C address of the sensor if not the default of 0x1D.
+    :param ~busio.I2C i2c: The I2C bus the device is connected to.
+    :param int address: The I2C device address. Defaults to :const:`0x1D`
+
+
+    **Quickstart: Importing and using the device**
+
+        Here is an example of using the :class:`MMA8451` class.
+        First you will need to import the libraries to use the sensor
+
+        .. code-block:: python
+
+            import board
+            import adafruit_mma8451
+
+        Once this is done you can define your `board.I2C` object and define your sensor object
+
+        .. code-block:: python
+
+            i2c = board.I2C()  # uses board.SCL and board.SDA
+            sensor = adafruit_mma8451.MMA8451(i2c)
+
+        Now you have access to the :attr:`acceleration` and :attr:`orientation` attributes
+
+        .. code-block:: python
+
+            acc_x, acc_y, acc_z = sensor.acceleration
+            orientation = sensor.orientation
+
     """
 
     # Class-level buffer to reduce allocations and fragmentation.
     # Note this is NOT thread-safe or re-entrant by design!
     _BUFFER = bytearray(6)
 
-    def __init__(self, i2c, *, address=_MMA8451_DEFAULT_ADDRESS):
+    def __init__(self, i2c: I2C, *, address: int = _MMA8451_DEFAULT_ADDRESS) -> None:
         self._device = i2c_device.I2CDevice(i2c, address)
         # Verify device ID.
         if self._read_u8(_MMA8451_REG_WHOAMI) != 0x1A:
@@ -95,7 +140,9 @@ class MMA8451:
         # Activate at max rate, low noise mode
         self._write_u8(_MMA8451_REG_CTRL_REG1, 0x01 | 0x04)
 
-    def _read_into(self, address, buf, count=None):
+    def _read_into(
+        self, address: int, buf: bytearray, count: Optional[int] = None
+    ) -> bytearray:
         # Read bytes from the specified address into the provided buffer.
         # If count is not specified (the default) the entire buffer is filled,
         # otherwise only count bytes are copied in.
@@ -109,12 +156,12 @@ class MMA8451:
         with self._device as i2c:
             i2c.write_then_readinto(bytes([address & 0xFF]), buf, in_end=count)
 
-    def _read_u8(self, address):
+    def _read_u8(self, address: int) -> int:
         # Read an 8-bit unsigned value from the specified 8-bit address.
         self._read_into(address, self._BUFFER, count=1)
         return self._BUFFER[0]
 
-    def _write_u8(self, address, val):
+    def _write_u8(self, address: int, val: int) -> None:
         # Write an 8-bit unsigned value to the specified 8-bit address.
         with self._device as i2c:
             self._BUFFER[0] = address & 0xFF
@@ -122,16 +169,18 @@ class MMA8451:
             i2c.write(self._BUFFER, end=2)
 
     @property
-    def range(self):
+    def range(self) -> int:
         """Get and set the range of the sensor.  Must be a value of:
-        - RANGE_8G: +/- 8g
-        - RANGE_4G: +/- 4g (the default)
-        - RANGE_2G: +/- 2g
+
+        * ``RANGE_8G``: +/- 8g
+        * ``RANGE_4G``: +/- 4g (the default)
+        * ``RANGE_2G``: +/- 2g
+
         """
         return self._read_u8(_MMA8451_REG_XYZ_DATA_CFG) & 0x03
 
     @range.setter
-    def range(self, val):
+    def range(self, val: int) -> None:
         assert 0 <= val <= 2
         reg1 = self._read_u8(_MMA8451_REG_CTRL_REG1)
         self._write_u8(_MMA8451_REG_CTRL_REG1, 0x00)  # deactivate
@@ -139,21 +188,23 @@ class MMA8451:
         self._write_u8(_MMA8451_REG_CTRL_REG1, reg1 | 0x01)  # activate
 
     @property
-    def data_rate(self):
+    def data_rate(self) -> int:
         """Get and set the data rate of the sensor.  Must be a value of:
-        - DATARATE_800HZ:   800Hz (the default)
-        - DATARATE_400HZ:   400Hz
-        - DATARATE_200HZ:   200Hz
-        - DATARATE_100HZ:   100Hz
-        - DATARATE_50HZ:     50Hz
-        - DATARATE_12_5HZ: 12.5Hz
-        - DATARATE_6_25HZ: 6.25Hz
-        - DATARATE_1_56HZ: 1.56Hz
+
+        * ``DATARATE_800HZ``:   800Hz (the default)
+        * ``DATARATE_400HZ``:   400Hz
+        * ``DATARATE_200HZ``:   200Hz
+        * ``DATARATE_100HZ``:   100Hz
+        * ``DATARATE_50HZ``:     50Hz
+        * ``DATARATE_12_5HZ``: 12.5Hz
+        * ``DATARATE_6_25HZ``: 6.25Hz
+        * ``DATARATE_1_56HZ``: 1.56Hz
+
         """
         return (self._read_u8(_MMA8451_REG_CTRL_REG1) >> 3) & _MMA8451_DATARATE_MASK
 
     @data_rate.setter
-    def data_rate(self, val):
+    def data_rate(self, val: int) -> None:
         assert 0 <= val <= 7
         ctl1 = self._read_u8(_MMA8451_REG_CTRL_REG1)
         self._write_u8(_MMA8451_REG_CTRL_REG1, 0x00)  # deactivate
@@ -162,11 +213,11 @@ class MMA8451:
         self._write_u8(_MMA8451_REG_CTRL_REG1, ctl1 | 0x01)  # activate
 
     @property
-    def acceleration(self):
+    def acceleration(self) -> Tuple[float, float, float]:
         # pylint: disable=no-else-return
         # This needs to be refactored when it can be tested
         """Get the acceleration measured by the sensor.  Will return a 3-tuple
-        of X, Y, Z axis acceleration values in m/s^2.
+        of X, Y, Z axis acceleration values in :math:`m/s^2`.
         """
         # Read 6 bytes for 16-bit X, Y, Z values.
         self._read_into(_MMA8451_REG_OUT_X_MSB, self._BUFFER, count=6)
@@ -199,15 +250,17 @@ class MMA8451:
             raise RuntimeError("Unexpected range!")
 
     @property
-    def orientation(self):
+    def orientation(self) -> int:
         """Get the orientation of the MMA8451.  Will return a value of:
-        - PL_PUF: Portrait, up, front
-        - PL_PUB: Portrait, up, back
-        - PL_PDF: Portrait, down, front
-        - PL_PDB: Portrait, down, back
-        - PL_LRF: Landscape, right, front
-        - PL_LRB: Landscape, right, back
-        - PL_LLF: Landscape, left, front
-        - PL_LLB: Landscape, left, back
+
+        * ``PL_PUF``: Portrait, up, front
+        * ``PL_PUB``: Portrait, up, back
+        * ``PL_PDF``: Portrait, down, front
+        * ``PL_PDB``: Portrait, down, back
+        * ``PL_LRF``: Landscape, right, front
+        * ``PL_LRB``: Landscape, right, back
+        * ``PL_LLF``: Landscape, left, front
+        * ``PL_LLB``: Landscape, left, back
+
         """
         return self._read_u8(_MMA8451_REG_PL_STATUS) & 0x07
